@@ -1,8 +1,15 @@
-__all__ = ['Device', 'EntryInfo', 'Client', 'from_container', 'load_devices',
-           'cache', 'HappiItem', 'OphydItem', 'SearchResult']
+__all__ = [
+    "Client",
+    "EntryInfo",
+    "HappiItem",
+    "OphydItem",
+    "SearchResult",
+    "cache",
+    "from_container",
+    "load_devices",
+]
 from ._version import get_versions
 from .client import Client, SearchResult
-from .device import Device
 from .item import EntryInfo, HappiItem, OphydItem
 from .loader import cache, from_container, load_devices
 
