@@ -2,22 +2,47 @@
 Base backend database options.
 """
 import logging
+from typing import Any, Dict, Generator, List
 
 logger = logging.getLogger(__name__)
 
 
+ItemMeta = Dict[str, Any]
+ItemMetaGen = Generator[ItemMeta, None, None]
+
+
 class _Backend:
-    """Base class for backend database."""
+    """
+    Base class for backend database.
+
+    Backends are internal to happi and are subject to change.  Users should
+    interact with the :class:`happi.Client`.
+    """
 
     @property
-    def all_devices(self):
-        """List of all device sub-dictionaries."""
+    def all_items(self) -> List[ItemMeta]:
+        """
+        List of all items in the database.
+
+        This base class ``_Backend`` provides no implementation and requires
+        that a subclass provide its customized implementation.
+        """
         raise NotImplementedError
 
-    def find(self, multiples=False, **kwargs):
+    def clear_cache(self) -> None:
+        """
+        Request to clear any cached data.
+
+        Optional implementation may be customized in subclass.
+        """
+
+    def find(self, multiples: bool = False, **kwargs) -> ItemMetaGen:
         """
         Find an instance or instances that matches the search criteria.
 
+        This base class ``_Backend`` provides no implementation and requires
+        that a subclass provide its customized implementation.
+
         Parameters
         ----------
         multiples : bool
@@ -28,10 +53,18 @@ class _Backend:
         """
         raise NotImplementedError
 
-    def save(self, _id, post, insert=True):
+    def save(
+        self,
+        _id: str,
+        post: Dict[str, Any],
+        insert: bool = True
+    ) -> None:
         """
         Save information to the database.
 
+        This base class ``_Backend`` provides no implementation and requires
+        that a subclass provide its customized implementation.
+
         Parameters
         ----------
         _id : str
@@ -54,10 +87,13 @@ class _Backend:
         """
         raise NotImplementedError
 
-    def delete(self, _id):
+    def delete(self, _id: str) -> None:
         """
         Delete a device instance from the database.
 
+        This base class ``_Backend`` provides no implementation and requires
+        that a subclass provide its customized implementation.
+
         Parameters
         ----------
         _id : str
