@@ -7,11 +7,13 @@ import math
 import os
 import os.path
 import re
+from typing import Any, Callable, Dict, List, Optional, Union
 
 import simplejson as json
 
+from .. import utils
 from ..errors import DuplicateError, SearchError
-from .core import _Backend
+from .core import ItemMeta, ItemMetaGen, _Backend
 
 logger = logging.getLogger(__name__)
 
@@ -48,27 +50,46 @@ class JSONBackend(_Backend):
         Path to JSON file.
     initialze : bool, optional
         Initialize a new empty JSON file to begin filling.
+    cfg_path : str, optional
+        Path to the happi config.
     """
 
-    def __init__(self, path, initialize=False):
-        self.path = os.path.expanduser(path)
+    def __init__(
+        self,
+        path: str,
+        initialize: bool = False,
+        cfg_path: Optional[str] = None
+    ) -> None:
+        self._load_cache: Dict[str, ItemMeta] = None
+        # Determine the cfg dir and build path to json db based on that unless we're initted w/o a config
+        if cfg_path is not None:
+            cfg_dir = os.path.dirname(cfg_path)
+            self.path = utils.build_abs_path(cfg_dir, path)
+        else:
+            self.path = path
         # Create a new JSON file if requested
         if initialize:
             self.initialize()
 
-    def _load_or_initialize(self):
-        """Load an existing database or initialize a new one."""
-        try:
-            return self.load()
-        except FileNotFoundError:
-            logger.debug('Initializing new database')
+    def clear_cache(self) -> None:
+        """Clear the loaded cache."""
+        self._load_cache = None
 
-        self.initialize()
-        return self.load()
+    def _load_or_initialize(self) -> Optional[Dict[str, ItemMeta]]:
+        """Load an existing database or initialize a new one."""
+        if self._load_cache is None:
+            try:
+                self._load_cache = self.load()
+            except FileNotFoundError:
+                logger.debug("Initializing new database")
+                self.initialize()
+                self._load_cache = self.load()
+
+        return self._load_cache
 
     @property
-    def all_devices(self):
-        """All of the devices in the database."""
+    def all_items(self) -> List[ItemMeta]:
+        """All of the items in the database."""
         json = self._load_or_initialize()
         return list(json.values())
 
@@ -95,7 +116,7 @@ class JSONBackend(_Backend):
         # Dump an empty dictionary
         self.store({})
 
-    def load(self):
+    def load(self) -> Dict[str, ItemMeta]:
         """Load the JSON database."""
         with open(self.path, 'r') as f:
             raw_json = f.read()
@@ -103,7 +124,7 @@ class JSONBackend(_Backend):
         # Allow for empty files to be considered valid databases:
         return json.loads(raw_json) if raw_json else {}
 
-    def store(self, db):
+    def store(self, db: Dict[str, ItemMeta]) -> None:
         """
         Stash the database in the JSON file.
 
@@ -131,14 +152,14 @@ class JSONBackend(_Backend):
                     # Release lock in filesystem
                     fcntl.flock(f, fcntl.LOCK_UN)
 
-    def _iterative_compare(self, comparison):
+    def _iterative_compare(self, comparison: Callable) -> ItemMetaGen:
         """
         Yields documents in which ``comparison(name, doc)`` returns `True`.
 
         Parameters
         ----------
-        comparison : callable
-            A comparison function with a signature of (device_id, doc).
+        comparison : Callable
+            A comparison function with a signature of (item_id, doc).
         """
 
         db = self._load_or_initialize()
@@ -152,12 +173,12 @@ class JSONBackend(_Backend):
             except Exception as ex:
                 logger.debug('Comparison method failed: %s', ex, exc_info=ex)
 
-    def get_by_id(self, id_):
-        """Get a device by ID if it exists, or return None."""
+    def get_by_id(self, id_: str) -> ItemMeta:
+        """Get an item by ID if it exists, or return None."""
         db = self._load_or_initialize()
         return db.get(id_)
 
-    def find(self, to_match):
+    def find(self, to_match: Dict[str, Any]) -> ItemMetaGen:
         """
         Find an instance or instances that matches the search criteria.
 
@@ -175,7 +196,14 @@ class JSONBackend(_Backend):
 
         yield from self._iterative_compare(comparison)
 
-    def find_range(self, key, *, start, stop=None, to_match):
+    def find_range(
+        self,
+        key: str,
+        *,
+        start: Union[int, float],
+        stop: Optional[Union[int, float]] = None,
+        to_match: Dict[str, Any]
+    ) -> ItemMetaGen:
         """
         Find an instance or instances that matches the search criteria, such
         that ``start <= entry[key] < stop``.
@@ -211,7 +239,12 @@ class JSONBackend(_Backend):
 
         yield from self._iterative_compare(comparison)
 
-    def find_regex(self, to_match, *, flags=re.IGNORECASE):
+    def find_regex(
+        self,
+        to_match: Dict[str, Any],
+        *,
+        flags=re.IGNORECASE
+    ) -> ItemMetaGen:
         """
         Find an instance or instances that matches the search criteria,
         using regular expressions.
@@ -233,56 +266,61 @@ class JSONBackend(_Backend):
 
         yield from self._iterative_compare(comparison)
 
-    def save(self, _id, post, insert=True):
+    def save(
+        self,
+        _id: str,
+        post: Dict[str, Any],
+        insert: bool = True
+    ) -> None:
         """
         Save information to the database.
 
         Parameters
         ----------
         _id : str
-            ID of device.
+            ID of item.
         post : dict
             Information to place in database.
         insert : bool, optional
-            Whether or not this is a new device to the database.
+            Whether or not this is a new item to the database.
 
         Raises
         ------
         DuplicateError
-            If ``insert`` is `True`, but there is already a device with the
+            If ``insert`` is `True`, but there is already an item with the
             provided ``_id``.
         SearchError
-            If ``insert`` is `False`, but there is no device with the provided
+            If ``insert`` is `False`, but there is no item with the provided
             ``_id``.
         PermissionError
             If the write operation fails due to issues with permissions.
         """
 
         with _load_and_store_context(self) as db:
-            # New device
+            # New item
             if insert:
                 if _id in db.keys():
-                    raise DuplicateError(f"Device {_id} already exists")
+                    raise DuplicateError(f"Item {_id} already exists")
                 # Add _id keyword
                 post.update({'_id': _id})
                 # Add to database
                 db[_id] = post
-            # Updating device
+            # Updating item
             else:
                 # Edit information
                 try:
                     db[_id].update(post)
                 except KeyError:
-                    raise SearchError("No device found {}".format(_id))
+                    raise SearchError("No item found {}".format(_id))
 
-    def delete(self, _id):
+    def delete(self, _id: str) -> None:
         """
-        Delete a device instance from the database.
+        Delete an item instance from the database.
 
         Parameters
         ----------
         _id : str
-            ID of device.
+            ID of item.
 
         Raises
         ------
