@@ -3,13 +3,14 @@ MongoDB Backend Implementation using the ``PyMongo`` package.
 """
 import logging
 import re
+from typing import Any, Dict, List, Optional, Union
 
 import bson.regex
 from pymongo import MongoClient
 from pymongo.errors import OperationFailure, ServerSelectionTimeoutError
 
 from ..errors import DatabaseError, DuplicateError, SearchError
-from .core import _Backend
+from .core import ItemMeta, ItemMetaGen, _Backend
 
 logger = logging.getLogger(__name__)
 
@@ -32,6 +33,8 @@ class MongoBackend(_Backend):
         Database name within the MongoDB instance.
     timeout : float, optional
         Time to wait for connection attempt.
+    cfg_path : str, optional
+        Path to the happi config.
     """
 
     _timeout = 5
@@ -39,7 +42,7 @@ class MongoBackend(_Backend):
 
     def __init__(self, host=None, user=None,
                  pw=None, db=None, collection=None,
-                 timeout=None):
+                 timeout=None, cfg_path=None):
         # Default timeout
         timeout = timeout or self._timeout
         # Format connection string
@@ -64,11 +67,11 @@ class MongoBackend(_Backend):
                                 'port specified at startup')
 
     @property
-    def all_devices(self):
-        """List of all device sub-dictionaries."""
-        return self._collection.find()
+    def all_items(self) -> List[ItemMeta]:
+        """List of all item sub-dictionaries."""
+        return list(self._collection.find())
 
-    def find(self, to_match):
+    def find(self, to_match: Dict[str, Any]) -> ItemMetaGen:
         """
         Yield all instances that match the given search criteria.
 
@@ -80,7 +83,14 @@ class MongoBackend(_Backend):
 
         yield from self._collection.find(to_match)
 
-    def find_range(self, key, *, start, stop=None, to_match):
+    def find_range(
+        self,
+        key: str,
+        *,
+        start: Union[int, float],
+        stop: Optional[Union[int, float]] = None,
+        to_match: Dict[str, Any]
+    ) -> ItemMetaGen:
         """
         Find an instance or instances that matches the search criteria, such
         that ``start <= entry[key] < stop``.
@@ -110,20 +120,24 @@ class MongoBackend(_Backend):
         match.update(**to_match)
         yield from self._collection.find(match)
 
-    def get_by_id(self, _id):
+    def get_by_id(self, _id: str) -> ItemMeta:
         """
-        Get a device by ID if it exists, or return None.
+        Get an item by ID if it exists, or return None.
 
         Parameters
         ----------
         _id : str
-            The device ID.
+            The item ID.
         """
-
         for item in self._collection.find({'_id': _id}):
             return item
 
-    def find_regex(self, to_match, *, flags=re.IGNORECASE):
+    def find_regex(
+        self,
+        to_match: Dict[str, Any],
+        *,
+        flags=re.IGNORECASE
+    ) -> ItemMetaGen:
         """
         Yield all instances that match the given search criteria.
 
@@ -144,26 +158,31 @@ class MongoBackend(_Backend):
 
         yield from self._collection.find(regexes)
 
-    def save(self, _id, post, insert=True):
+    def save(
+        self,
+        _id: str,
+        post: Dict[str, Any],
+        insert: bool = True
+    ) -> None:
         """
         Save information to the database.
 
         Parameters
         ----------
         _id : str
-            ID of device.
+            ID of item.
         post : dict
             Information to place in database.
         insert : bool, optional
-            Whether or not this a new device to the database.
+            Whether or not this a new item to the database.
 
         Raises
         ------
         DuplicateError
-            If ``insert`` is `True`, but there is already a device with the
+            If ``insert`` is `True`, but there is already an item with the
             provided ``_id``.
         SearchError
-            If ``insert`` is `False`, but there is no device with the provided
+            If ``insert`` is `False`, but there is no item with the provided
             ``_id``.
         PermissionError
             If the write operation fails due to issues with permissions.
@@ -175,29 +194,35 @@ class MongoBackend(_Backend):
                                                  {'$set': post},
                                                  upsert=insert)
         except OperationFailure:
-            raise PermissionError("Unauthorized command, make sure you are "
-                                  "using a user with write permissions")
+            raise PermissionError(
+                "Unauthorized command, make sure you are "
+                "using a user with write permissions"
+            )
 
         if insert and not result.upserted_id:
-            raise DuplicateError('Device with id {} has already been entered '
-                                 'into the database, use load_device and '
-                                 'save if you wish to make changes to the '
-                                 'device'.format(_id))
+            raise DuplicateError(
+                "Item with id {} has already been entered "
+                "into the database, use load_item and "
+                "save if you wish to make changes to the "
+                "item".format(_id)
+            )
 
         if not insert and result.matched_count == 0:
-            raise SearchError('No device found with id {} please, if this is '
-                              'a new device, try add_device. If not, make '
-                              'sure that the device information being sent is '
-                              'correct'.format(_id))
+            raise SearchError(
+                "No item found with id {} please, if this is "
+                "a new item, try add_item. If not, make "
+                "sure that the item information being sent is "
+                "correct".format(_id)
+            )
 
-    def delete(self, _id):
+    def delete(self, _id: str) -> None:
         """
-        Delete a device instance from the database.
+        Delete an item instance from the database.
 
         Parameters
         ----------
         _id : str
-            ID of device.
+            ID of item.
         """
 
         res = self._collection.delete_one({'_id': _id})
