@@ -1,5 +1,7 @@
-import collections
+import collections.abc
 import configparser
+import contextlib
+import copy
 import inspect
 import itertools
 import logging
@@ -7,8 +9,7 @@ import os
 import re
 import sys
 import time as ttime
-import warnings
-from typing import Any, Dict, Optional, Type
+from typing import Any, Dict, List, Optional, Sequence, Type
 
 from . import containers
 from .backends import BACKENDS, DEFAULT_BACKEND
@@ -25,7 +26,7 @@ def _looks_like_database(obj):
     return (isinstance(obj, _Backend) or
             all(
                 hasattr(obj, attr) for attr in (
-                    'find', 'all_devices', 'delete', 'save')
+                    'find', 'all_items', 'delete', 'save')
                 )
             )
 
@@ -53,22 +54,17 @@ class SearchResult(collections.abc.Mapping):
         The resulting HappiItem metadata.
     """
 
-    def __init__(self, client, device):
-        self._device = device
+    def __init__(self, client, item):
+        self._item = item
         self._instantiated = None
         self.client = client
-        self.metadata = device.post()
-
-    @property
-    def device(self):
-        warnings.warn('SearchResult.device deprecated, use SearchResult.item')
-        return self.item
+        self.metadata = item.post()
 
     @property
     def item(self):
-        if self._device is None:
-            self._device = self.client.find_device(**self.metadata)
-        return self._device
+        if self._item is None:
+            self._item = self.client.find_item(**self.metadata)
+        return self._item
 
     def get(self, attach_md=True, use_cache=True, threaded=False):
         '''(get) ''' + from_container.__doc__
@@ -94,6 +90,16 @@ class SearchResult(collections.abc.Mapping):
             f'metadata={self.metadata})'
         )
 
+    def __eq__(self, __o: object) -> bool:
+        if isinstance(__o, self.__class__):
+            return ((self.client.backend == __o.client.backend)
+                    and (self['_id'] == __o['_id']))
+        else:
+            return False
+
+    def __hash__(self) -> int:
+        return hash((self.client.backend, self['_id']))
+
 
 class Client(collections.abc.Mapping):
     """
@@ -118,6 +124,8 @@ class Client(collections.abc.Mapping):
     _results_wrap_class = SearchResult
 
     def __init__(self, database=None, **kwargs):
+        self._retain_cache = False
+
         # Use supplied backend
         if database:
             self.backend = database
@@ -134,18 +142,37 @@ class Client(collections.abc.Mapping):
                     f"Failed to instantiate a {DEFAULT_BACKEND} backend"
                 ) from exc
 
+    @contextlib.contextmanager
+    def retain_cache_context(self, clear_first: bool = True):
+        """
+        Context manager which can be used to retain the happi backend cache.
+
+        Parameters
+        ----------
+        clear_first : bool, optional
+            Clear the cache before entering the block.  Defaults to True.
+        """
+        if clear_first:
+            self.backend.clear_cache()
+
+        try:
+            self._retain_cache = True
+            yield
+        finally:
+            self._retain_cache = False
+
     def find_document(self, **kwargs):
         """
-        Load a device document from the database.
+        Load an item document from the database.
 
         If multiple matches are found, a single document will be returned to
-        the user. How the database will choose to select this device is based
+        the user. How the database will choose to select this item is based
         on each individual implementation
 
         Parameters
         ----------
         kwargs
-            Keyword pairs of information used to locate the device
+            Keyword pairs of information used to locate the item
 
         Returns
         -------
@@ -159,115 +186,145 @@ class Client(collections.abc.Mapping):
 
         See Also
         --------
-        :meth:`.find_device`, :meth:`.search`
+        :meth:`.find_item`, :meth:`.search`
         """
 
         if len(kwargs) == 0:
-            raise SearchError('No information pertinent to device given')
+            raise SearchError("No information pertinent to item given")
 
+        self._maybe_clear_cache()
         matches = list(itertools.islice(self.backend.find(kwargs), 1))
         if not matches:
             raise SearchError(
-                'No device information found that matches the search criteria')
-        return matches[0]
+                "No item information found that matches the search criteria"
+            )
+        return copy.deepcopy(matches[0])
 
-    def create_device(self, device_cls, **kwargs):
+    def create_item(self, item_cls, **kwargs):
         """
         Instantiate a HappiItem from its container class and keyword arguments.
 
         Parameters
         ----------
-        device_cls : class
+        item_cls : type
             The HappiItem container class to instantiate.
-        kwargs
-            Information to pass through to the device, upon initialization.
+
+        **kwargs :
+            Information to pass through to the item, upon initialization.
 
         Returns
         -------
-        device : object
-            An instantiated version of the device.
+        item : object
+            An instantiated version of the item.
 
         Raises
         ------
         TypeError
             If the provided class is not a subclass of :class:`.HappiItem`.
 
-
         Example
         -------
         .. code::
 
-            device = client.create_device(Device,   name='my_device' ...)
-            device = client.create_device('Device', name='my_device',...)
-
+            item = client.create_item(Item, name='my_item' ...)
+            item = client.create_item('Item', name='my_item',...)
         """
 
-        # If specified by a string
-        if device_cls in containers.registry:
-            device_cls = containers.registry[device_cls]
+        # string -> class, if in the registry
+        if item_cls in containers.registry:
+            item_cls = containers.registry[item_cls]
 
         # Check that this is a valid HappiItem
-        if isinstance(device_cls, str):
+        if isinstance(item_cls, str):
             raise TypeError(
-                f'The device class {device_cls!r} is not in the registry'
+                f'The container class {item_cls!r} is not in the registry'
             )
 
-        if (not inspect.isclass(device_cls) or
-                not issubclass(device_cls, HappiItem)):
-            raise TypeError(f'{device_cls!r} is not a subclass of HappiItem')
+        if not (inspect.isclass(item_cls) and issubclass(item_cls, HappiItem)):
+            raise TypeError(f"{item_cls!r} is not a subclass of HappiItem")
 
-        device = device_cls(**kwargs)
-        # Add the method to the device
+        item = item_cls(**kwargs)
 
-        def save_device():
-            self.add_device(device)
+        def save_item():
+            self.add_item(item)
 
-        device.save = save_device
-        return device
+        # Add the method to the item
+        item.save = save_item
+        return item
 
-    def add_device(self, device):
+    def add_item(self, item):
         """
-        Add a device into the database.
+        Add an item into the database.
 
         Parameters
         ----------
-        device : :class:`.HappiItem`
-            The device to store in the database.
+        item : :class:`.HappiItem`
+            The item to store in the database.
 
         Returns
         -------
         _id : str
-            The id of the device added.
+            The id of the item added.
 
         Raises
         ------
         EntryError
-            If all of the mandatory information for the device has not been
-            specified or there is already a device with that ``id`` in the
+            If all of the mandatory information for the item has not been
+            specified or there is already an item with that ``id`` in the
             database.
         """
 
-        logger.info("Storing device %r ...", device)
-        _id = self._store(device, insert=True)
-        logger.info('HappiItem %r has been succesfully added to the '
-                    'database', device)
+        logger.info("Storing item %r ...", item)
+        _id = self._store(item, insert=True)
+        logger.info(
+            "HappiItem %r has been succesfully added to the database",
+            item
+        )
 
-        def save_device():
-            self._store(device, insert=False)
+        def save_item():
+            self._store(item, insert=False)
 
-        device.save = save_device
+        item.save = save_item
         return _id
 
-    def find_device(self, **post):
+    def _get_item_from_document(self, doc: Dict[str, Any]) -> HappiItem:
         """
-        Query the database for an individual HappiItem.
-
-        If multiple devices are found, only the first is returned.
+        Create a HappiItem given its backend-provided document.
 
         Parameters
         ----------
-        post
-            Key-value pairs of search criteria used to find the device.
+        doc : dict
+            Document from the backend.
+
+        Returns
+        -------
+        item : :class:`.HappiItem`
+            An item that matches the characteristics given.
+        """
+        try:
+            item = self.create_item(doc["type"], **doc)
+        except (KeyError, TypeError) as exc:
+            raise EntryError(
+                "The information relating to the container class "
+                "has been modified to the point where the object "
+                "can not be initialized, please load the "
+                "corresponding document"
+            ) from exc
+
+        # Add the save method to the item
+        item.save = lambda: self._store(item, insert=False)
+        return item
+
+    def find_item(self, **post):
+        """
+        Query the database for an individual HappiItem.
+
+        If multiple items are found, only the first is returned.
+
+        Parameters
+        ----------
+        **post :
+            Key-value pairs of search criteria used to find the item.
 
         Raises
         ------
@@ -276,45 +333,30 @@ class Client(collections.abc.Mapping):
 
         Returns
         -------
-        device : :class:`.HappiItem`
-            A device that matches the characteristics given.
+        item : :class:`.HappiItem`
+            A HappiItem instance for the document.
         """
-
-        logger.debug("Gathering information about the device ...")
-        doc = self.find_document(**post)
-        # Instantiate HappiItem
-        logger.debug("Instantiating device based on found information ...")
-        try:
-            device = self.create_device(doc['type'], **doc)
-        except (KeyError, TypeError) as exc:
-            raise EntryError('The information relating to the device class '
-                             'has been modified to the point where the object '
-                             'can not be initialized, please load the '
-                             'corresponding document') from exc
-
-        # Add the save method to the device
-        device.save = lambda: self._store(device, insert=False)
-        return device
+        return self._get_item_from_document(self.find_document(**post))
 
     def load_device(self, use_cache=True, **post):
         """
-        Find a device in the database and instantiate it.
+        Find an item in the database and instantiate it.
 
         Essentially a shortcut for:
 
         .. code:: python
 
-            container = client.find_device(name='...')
-            device = from_container(container)
+            container = client.find_item(name='...')
+            item = from_container(container)
 
         Parameters
         ----------
         post
             Key-value pairs of search criteria passed to
-            :meth:`.Client.find_device`.
+            :meth:`.Client.find_item`.
 
         use_cache : bool, optional
-            Choice to use a cached device. See :meth:`.from_container` for more
+            Choice to use a cached item. See :meth:`.from_container` for more
             information.
 
         Returns
@@ -322,8 +364,7 @@ class Client(collections.abc.Mapping):
         object
             Instantiated object.
         """
-
-        cntr = self.find_device(**post)
+        cntr = self.find_item(**post)
         return from_container(cntr, use_cache=use_cache)
 
     def change_container(
@@ -410,60 +451,48 @@ class Client(collections.abc.Mapping):
 
     def validate(self):
         """
-        Validate all of the devices in the database.
+        Validate all of the items in the database.
 
-        This is done by attempting to initialize each device and asserting
+        This is done by attempting to initialize each item and asserting
         their mandatory information is present. Information is written to the
         logger.
 
         Returns
         -------
         ids : list of str
-            List of device ids that have failed verification.
+            List of item ids that have failed verification.
         """
 
         bad = list()
         logger.debug('Loading database to validate contained devices ...')
-        for post in self.backend.all_devices:
+        for doc in self.backend.all_items:
             # Try and load device based on database info
+            _id = doc.get(self._id_key, "(unknown id)")
             try:
-                # HappiItem identification
-                _id = post[self._id_key]
-                logger.debug('Attempting to initialize %s...', _id)
-                # Load HappiItem
-                device = self.find_device(**post)
-                logger.debug('Attempting to validate ...')
-                self._validate_device(device)
-            except KeyError:
-                logger.error("Post has no id  %s", post)
-            # Log all generated exceptions
+                logger.debug("Attempting to initialize %s...", _id)
+                item = self._get_item_from_document(doc)
+                logger.debug("Attempting to validate ...")
+                self._validate_item(item)
             except Exception as e:
                 logger.warning("Failed to validate %s because %s", _id, e)
                 bad.append(_id)
-            # Report successes
             else:
                 logger.debug('Successfully validated %s', _id)
         return bad
 
-    @property
-    def all_devices(self):
-        """A list of all contained devices."""
-        warnings.warn('Client.all_devices deprecated, use all_items instead')
-        return self.all_items
-
     @property
     def all_items(self):
         """A list of all contained items."""
         return [res.item for res in self.search()]
 
     def __getitem__(self, key):
-        """Get a device ID."""
+        """Get an item ID."""
         try:
-            device = self.find_device(**self.backend.get_by_id(key))
+            item = self._get_item_from_document(self.backend.get_by_id(key))
         except Exception as ex:
             raise KeyError(key) from ex
 
-        return SearchResult(client=self, device=device)
+        return SearchResult(client=self, item=item)
 
     def __iter__(self):
         for info in self.backend.find({}):
@@ -472,40 +501,57 @@ class Client(collections.abc.Mapping):
     def __len__(self):
         return len(self.all_items)
 
-    def _get_search_results(self, items, *, wrap_cls=None):
+    def _maybe_clear_cache(self):
+        """Clear the backend cache if not in a retain-cache block."""
+        if not self._retain_cache:
+            self.backend.clear_cache()
+
+    def _get_search_results(
+        self, docs: Sequence[Dict[str, Any]], *, wrap_cls: Optional[type] = None
+    ) -> List[SearchResult]:
         """
         Return search results to the user, optionally wrapping with a class.
         """
         wrap_cls = wrap_cls or self._results_wrap_class
+        assert wrap_cls is not None
+
         results = []
-        for info in items:
+        for doc in docs:
             try:
-                result = wrap_cls(client=self, device=self.find_device(**info))
-                results.append(result)
+                results.append(
+                    wrap_cls(
+                        client=self,
+                        item=self._get_item_from_document(doc)
+                    )
+                )
             except Exception as exc:
-                logger.warning('Entry for %s is malformed (%s). Skipping.',
-                               info['name'], exc)
+                logger.warning(
+                    "Entry for %s is malformed (%s). Skipping.",
+                    doc["name"], exc
+                )
         return results
 
-    def search_range(self, key, start, end=None, **kwargs):
+    def search_range(
+        self, key: str, start: float, end: Optional[float] = None, **kwargs
+    ) -> List[SearchResult]:
         """
-        Search the database for a device or devices using a numerical range.
+        Search the database for an item or items using a numerical range.
 
         Parameters
         -----------
         key : str
             Database key to search.
         start : float, optional
-            Minimum beamline position to include devices.
+            Minimum beamline position to include items.
         end : float, optional
-            Maximum beamline position to include devices.
+            Maximum beamline position to include items.
         **kwargs : optional
             Additional information used to filter through the database
             structured as key-value pairs for the desired pieces of EntryInfo.
 
         Returns
         -------
-        list of devices or dictionaries
+        list of SearchResult
 
         Example
         -------
@@ -516,13 +562,14 @@ class Client(collections.abc.Mapping):
                                               beamline='HXR')
         """
 
+        self._maybe_clear_cache()
         items = self.backend.find_range(key, start=start, stop=end,
                                         to_match=kwargs)
         return self._get_search_results(items)
 
     def search(self, **kwargs):
         """
-        Search the database for a device or devices.
+        Search the database for item(s).
 
         Parameters
         -----------
@@ -543,12 +590,15 @@ class Client(collections.abc.Mapping):
             hxr_valves  = client.search(type='Valve', beamline='HXR')
         """
 
+        self._maybe_clear_cache()
         items = self.backend.find(kwargs)
         return self._get_search_results(items)
 
-    def search_regex(self, flags=re.IGNORECASE, **kwargs):
+    def search_regex(
+        self, flags=re.IGNORECASE, **kwargs
+    ) -> List[SearchResult]:
         """
-        Search the database for a device or devices.
+        Search the database for items(s).
 
         Parameters
         -----------
@@ -562,7 +612,7 @@ class Client(collections.abc.Mapping):
 
         Returns
         -------
-        list of devices or dictionaries
+        list of SearchResult
 
         Example
         -------
@@ -572,6 +622,7 @@ class Client(collections.abc.Mapping):
             three_valves = client.search_regex(_id='VALVE[123]')
         """
 
+        self._maybe_clear_cache()
         items = self.backend.find_regex(kwargs, flags=flags)
         return self._get_search_results(items)
 
@@ -592,7 +643,7 @@ class Client(collections.abc.Mapping):
         # Load documents
         devs = self.all_items
         logger.info('Creating file at %s ...', path)
-        # Load device information
+        # Load item information
         with path as f:
             for dev in devs:
                 try:
@@ -603,52 +654,52 @@ class Client(collections.abc.Mapping):
                     logger.error("HappiItem %s was missing attribute %s",
                                  dev.name, e)
 
-    def remove_device(self, device):
+    def remove_item(self, item):
         """
-        Remove a device from the database.
+        Remove an item from the database.
 
         Parameters
         ----------
-        device : :class:`.HappiItem`
+        item : :class:`.HappiItem`
             HappiItem to be removed from the database.
         """
 
         # HappiItem Check
-        if not isinstance(device, HappiItem):
+        if not isinstance(item, HappiItem):
             raise ValueError("Must supply an object of type `HappiItem`")
         logger.info("Attempting to remove %r from the "
-                    "collection ...", device)
-        # Check that device is in the database
-        _id = getattr(device, self._id_key)
+                    "collection ...", item)
+        # Check that item is in the database
+        _id = getattr(item, self._id_key)
         self.backend.delete(_id)
 
-    def _validate_device(self, device):
-        """Validate that a device has all of the mandatory information."""
-        logger.debug('Validating device %r ...', device)
+    def _validate_item(self, item):
+        """Validate that an item has all of the mandatory information."""
+        logger.debug('Validating item %r ...', item)
 
         # Check type
-        if not isinstance(device, HappiItem):
-            raise ValueError('{!r} is not a subclass of '
-                             'HappiItem'.format(device))
+        if not isinstance(item, HappiItem):
+            raise ValueError(f"{item!r} is not a subclass of HappiItem")
         logger.debug('Checking mandatory information has been entered ...')
         # Check that all mandatory info has been entered
-        missing = [info.key for info in device.entry_info
+        missing = [info.key for info in item.entry_info
                    if not info.optional and
-                   info.default == getattr(device, info.key)]
+                   info.default == getattr(item, info.key)]
         # Abort initialization if missing mandatory info
         if missing:
-            raise EntryError('Missing mandatory information ({}) for {}'
-                             ''.format(', '.join(missing),
-                                       device.__class__.__name__))
-        logger.debug('HappiItem %r has been validated.', device)
+            raise EntryError(
+                "Missing mandatory information ({}) for {}"
+                "".format(", ".join(missing), item.__class__.__name__)
+            )
+        logger.debug('HappiItem %r has been validated.', item)
 
-    def _store(self, device, insert=False):
+    def _store(self, item, insert=False):
         """
         Store a document in the database.
 
         Parameters
         ----------
-        post : :class:`.HappiItem`
+        item : :class:`.HappiItem`
             HappiItem to save.
         insert : bool, optional
             Set to `True` if this is a new entry.
@@ -658,19 +709,19 @@ class Client(collections.abc.Mapping):
         DuplicateError
             When ``_id`` already exists.
         EntryError
-            If the device doesn't the correct information.
+            If the item doesn't the correct information.
 
         Todo
         ----
         Enforce parent is an already entered name
         """
 
-        logger.debug('Loading a device into the collection ...')
+        logger.debug('Loading an item into the collection ...')
 
-        # Validate device is ready for storage
-        self._validate_device(device)
-        # Grab information from device
-        post = device.post()
+        # Validate item is ready for storage
+        self._validate_item(item)
+        # Grab information from item
+        post = item.post()
         # save the old name in case the user is trying to
         # change the 'name' of an entry
         the_old_name = post.get('_id', None)
@@ -678,15 +729,18 @@ class Client(collections.abc.Mapping):
         creation = post.get('creation', ttime.ctime())
         # Clean supplied information
         [post.pop(key) for key in self._client_attrs if key in post]
-        # Note that device has some unrecognized metadata
+        # Note that item has some unrecognized metadata
         for key in post.keys():
-            if key not in device.info_names:
-                logger.debug("HappiItem %r defines an extra piece of "
-                             "information under the keyword %s",
-                             device, key)
+            if key not in item.info_names:
+                logger.debug(
+                    "HappiItem %r defines an extra piece of "
+                    "information under the keyword %s",
+                    item,
+                    key,
+                )
         # Add metadata from the Client Side
 
-        tpe = containers.registry.entry_for_class(device.__class__)
+        tpe = containers.registry.entry_for_class(item.__class__)
         post.update({'type': tpe,
                      'creation': creation,
                      'last_edit': ttime.ctime()})
@@ -747,10 +801,40 @@ class Client(collections.abc.Mapping):
         cfg_parser = configparser.ConfigParser()
         cfg_file = cfg_parser.read(cfg)
         logger.debug("Loading configuration file at %r", cfg_file)
-        db_kwargs = cfg_parser['DEFAULT']
+
+        # Gather the backend from each section
+        sub_backends = []
+        for section in cfg_parser.keys():
+            bknd = cls._get_backend_from_config(cfg_parser[section], cfg)
+            if bknd:
+                sub_backends.append(bknd)
+
+        if len(sub_backends) == 0:
+            raise RuntimeError('No valid backends loaded')
+        elif len(sub_backends) == 1:
+            # return single backend client
+            return cls(database=sub_backends[0])
+        else:
+            # multi backend configuration have multiple sections
+            multi_backend = BACKENDS['multi']
+
+            # move DEFAULT from index 0 to -1, if it exists
+            if dict(cfg_parser['DEFAULT']):
+                default_backend = sub_backends.pop(0)
+                sub_backends.append(default_backend)
+
+            # return client with multi_backend
+            db = multi_backend(backends=sub_backends)
+            return cls(database=db)
+
+    @staticmethod
+    def _get_backend_from_config(
+        cfg_section: configparser.SectionProxy,
+        cfg: str
+    ) -> _Backend:
         # If a backend is specified use it, otherwise default
-        if 'backend' in db_kwargs:
-            db_str = db_kwargs.pop('backend')
+        if 'backend' in cfg_section:
+            db_str = cfg_section.pop('backend')
             try:
                 backend = BACKENDS[db_str]
             except KeyError:
@@ -762,19 +846,23 @@ class Client(collections.abc.Mapping):
 
         logger.debug(
             "Using Happi backend %r with kwargs %s",
-            backend, db_kwargs
+            backend, cfg_section
         )
 
+        if not dict(cfg_section):
+            # section is empty, skip
+            return
+
         # Create our database with provided kwargs
         try:
-            database = backend(**db_kwargs)
-            return cls(database=database)
+            database = backend(**cfg_section, cfg_path=cfg)
+            return database
         except Exception as ex:
             raise RuntimeError(
                 f'Unable to instantiate the client. Please verify that '
                 f'your HAPPI_CFG points to the correct file and has '
                 f'the required configuration settings. In {cfg!r}, found '
-                f'settings: {dict(db_kwargs)}.'
+                f'settings: {dict(cfg_section)}.'
             ) from ex
 
     @staticmethod
@@ -832,7 +920,7 @@ class Client(collections.abc.Mapping):
         Raises
         ------
         SearchError
-            If no devices in the database have an entry for the given field.
+            If no items in the database have an entry for the given field.
 
         Returns
         -------
