@@ -1,9 +1,13 @@
 # test_cli.py
 
+import functools
+import itertools
 import logging
 import re
+from typing import Any, Iterable, List, Tuple
 from unittest import mock
 
+import click
 import IPython
 import pytest
 from click.testing import CliRunner
@@ -12,117 +16,78 @@ import happi
 from happi.cli import happi_cli, search
 from happi.errors import SearchError
 
+try:
+    import pcdsutils.profile
+    if pcdsutils.profile.has_line_profiler:
+        test_line_prof = True
+    else:
+        test_line_prof = False
+except ImportError:
+    test_line_prof = False
 
-@pytest.fixture(scope='function')
-def happi_cfg(tmp_path, db):
-    happi_cfg_path = tmp_path / 'happi.cfg'
-    happi_cfg_path.write_text(f"""\
-[DEFAULT]'
-backend=json
-path={db}
-""")
-    return str(happi_cfg_path.absolute())
+
+logger = logging.getLogger(__name__)
 
 
 @pytest.fixture(scope='function')
-def db(tmp_path):
-    json_path = tmp_path / 'db.json'
-    json_path.write_text("""\
-{
-    "TST_BASE_PIM": {
-        "_id": "TST_BASE_PIM",
-        "active": true,
-        "args": [
-            "{{prefix}}"
-        ],
-        "beamline": "TST",
-        "creation": "Tue Jan 29 09:46:00 2019",
-        "device_class": "types.SimpleNamespace",
-        "kwargs": {
-            "name": "{{name}}"
-        },
-        "last_edit": "Thu Apr 12 14:40:08 2018",
-        "macros": null,
-        "name": "tst_base_pim",
-        "parent": null,
-        "prefix": "TST:BASE:PIM",
-        "screen": null,
-        "stand": "BAS",
-        "system": "diagnostic",
-        "type": "OphydItem",
-        "z": 3.0,
-        "y": 40.0
-    },
-    "TST_BASE_PIM2": {
-        "_id": "TST_BASE_PIM2",
-        "active": true,
-        "args": [
-            "{{prefix}}"
-        ],
-        "beamline": "TST",
-        "creation": "Wed Jan 30 09:46:00 2019",
-        "device_class": "types.SimpleNamespace",
-        "kwargs": {
-            "name": "{{name}}"
-        },
-        "last_edit": "Fri Apr 13 14:40:08 2018",
-        "macros": null,
-        "name": "tst_base_pim2",
-        "parent": null,
-        "prefix": "TST:BASE:PIM2",
-        "screen": null,
-        "stand": "BAS",
-        "system": "diagnostic",
-        "type": "OphydItem",
-        "z": 6.0,
-        "y": 10.0
-    }
-}
-""")
-    return str(json_path.absolute())
-
-
-@pytest.fixture(scope='function')
-def runner():
-    return CliRunner()
-
-
-@pytest.fixture(scope='function')
-def client(happi_cfg):
+def client(happi_cfg: str):
     return happi.client.Client.from_config(cfg=happi_cfg)
 
 
-def trim_split_output(strings, delim='\n'):
+def trim_split_output(strings: str, delim: str = '\n'):
     """
     Trim output of timestamped messages and registry lists.
     Split on delimiter (newline by default)
     """
     date_pattern = r"\[(\d{4})[-](0[1-9]|1[012])[-].*\]"
 
-    bad_substrs = [r"^pcdsdevices"]
+    ok_substrs = [r"happi.item."]
 
     # remove registry items
     new_out = [
         st for st in strings.split(delim)
-        if not any([re.search(substr, st) for substr in bad_substrs])
+        if any([re.search(substr, st) for substr in ok_substrs])
     ]
 
-    # string date-time from logging messages
+    # strip date-time from logging messages
     new_out = [re.sub(date_pattern, '', st) for st in new_out]
     return new_out
 
 
-def assert_match_expected(result, expected_output):
-    """standard checks for a cli result, confirms output matches expected"""
-    assert result.exit_code == 0
-    assert not result.exception
+def assert_match_expected(
+    result: click.testing.Result,
+    expected_output: Iterable[str],
+    expected_return: int = 0
+):
+    """
+    standard checks for a cli result, confirms output matches expected
+    SystemExit exception is acceptable if the expected return code is some failure code
+    """
+    logger.debug(result.output)
+    assert result.exit_code == expected_return
+    assert not result.exception or (isinstance(result.exception, SystemExit) and expected_return != 0)
 
     trimmed_output = trim_split_output(result.output)
     for message, expected in zip(trimmed_output, expected_output):
         assert message == expected
 
 
-def test_cli_no_argument(runner):
+def assert_in_expected(
+    result: click.testing.Result,
+    expected_inclusion: str,
+    expected_return: int = 0
+):
+    """
+    standard checks for a cli result, confirms output matches expected
+    SystemExit exception is acceptable if the expected return code is some failure code
+    """
+    logger.debug(result.output)
+    assert result.exit_code == expected_return
+    assert not result.exception or (isinstance(result.exception, SystemExit) and expected_return != 0)
+    assert expected_inclusion in result.output
+
+
+def test_cli_no_argument(runner: CliRunner):
     result = runner.invoke(happi_cli)
     assert result.exit_code == 0
     assert result.exception is None
@@ -131,7 +96,7 @@ def test_cli_no_argument(runner):
     assert 'Commands:' in result.output
 
 
-def test_search(client):
+def test_search(client: happi.client.Client):
     res = client.search_regex(beamline="TST")
 
     with search.make_context('search', ['beamline=TST'], obj=client) as ctx:
@@ -140,7 +105,11 @@ def test_search(client):
     assert [r.item for r in res] == [r.item for r in res_cli]
 
 
-def test_search_with_name(client, runner, happi_cfg):
+def test_search_with_name(
+    client: happi.client.Client,
+    runner: CliRunner,
+    happi_cfg: str
+):
     res = client.search_regex(name='TST_BASE_PIM2')
 
     with search.make_context('search', ['TST_BASE_PIM2'], obj=client) as ctx:
@@ -156,7 +125,7 @@ def test_search_with_name(client, runner, happi_cfg):
     assert 'duplicate search criteria' in bad_result.output
 
 
-def test_search_glob_regex(runner, happi_cfg):
+def test_search_glob_regex(runner: CliRunner, happi_cfg: str):
     glob_result = runner.invoke(happi_cli, ['--path', happi_cfg, 'search',
                                 '--names', 'tst_*2'])
 
@@ -166,7 +135,7 @@ def test_search_glob_regex(runner, happi_cfg):
     assert glob_result.output == regex_result.output
 
 
-def test_search_z(client):
+def test_search_z(client: happi.client.Client):
     res = client.search_regex(z="6.0")
     with search.make_context('search', ['z=6.0'], obj=client) as ctx:
         res_cli = search.invoke(ctx)
@@ -174,7 +143,11 @@ def test_search_z(client):
     assert [r.item for r in res] == [r.item for r in res_cli]
 
 
-def test_search_z_range(client, runner, happi_cfg):
+def test_search_z_range(
+    client: happi.client.Client,
+    runner: CliRunner,
+    happi_cfg: str
+):
     res = client.search_range('z', 3.0, 6.0)
 
     with search.make_context('search', ['z=3.0,6.0'], obj=client) as ctx:
@@ -196,22 +169,45 @@ def test_search_z_range(client, runner, happi_cfg):
     assert bad_result.exit_code == 1
     assert "Invalid range, make sure start < stop" in bad_result.output
 
-    # test conflicting ranges (should return no devices)
+    # test conflicting ranges (should return no items)
     conflict_result = runner.invoke(happi_cli, ['--path', happi_cfg,
                                     'search', 'y=1,3', 'z=3.0,6.0'])
 
     assert conflict_result.exit_code == 0
-    assert 'No devices found' in conflict_result.output
+    assert 'No items found' in conflict_result.output
 
     # test conflicting ranges but with opposite order
     conflict_result = runner.invoke(happi_cli, ['--path', happi_cfg,
                                     'search', 'z=3.0,6.0', 'y=1,3'])
 
     assert conflict_result.exit_code == 0
-    assert 'No devices found' in conflict_result.output
+    assert 'No items found' in conflict_result.output
 
 
-def test_both_range_and_regex_search(client):
+def test_search_int_float(runner: CliRunner, happi_cfg: str):
+    int_result = runner.invoke(happi_cli, ['--path', happi_cfg,
+                               'search', '--names', 'z=3'])
+    float_result = runner.invoke(happi_cli, ['--path', happi_cfg,
+                                 'search', '--names', 'z=3.0'])
+    assert int_result.output == float_result.output
+
+    # # TODO: add this test case once edit works on extraneous info
+    # edit_result = runner.invoke(
+    #     happi_cli,
+    #     ['-v', '--path', happi_cfg, 'edit', 'tst_base_pim', 'z=3.001'],
+    #     input='y'
+    # )
+    # assert edit_result.exit_code == 0
+
+    # int_result = runner.invoke(happi_cli, ['--path', happi_cfg,
+    #                            'search', '--names', 'z=3'])
+    # float_result = runner.invoke(happi_cli, ['--path', happi_cfg,
+    #                              'search', '--names', 'z=3.0'])
+    # assert int_result.output == ''
+    # assert float_result.output == ''
+
+
+def test_both_range_and_regex_search(client: happi.client.Client):
     # we're only interested in getting this entry (TST_BASE_PIM2)
     res = client.search_regex(z='6.0')
     # we're going to search for z=3,7 name=TST_BASE_PIM2
@@ -223,7 +219,7 @@ def test_both_range_and_regex_search(client):
     assert [r.item for r in res] == [r.item for r in res_cli]
 
 
-def test_search_json(runner, happi_cfg):
+def test_search_json(runner: CliRunner, happi_cfg: str):
     expected_output = '''[
   {
     "name": "tst_base_pim2",
@@ -260,7 +256,7 @@ def test_search_json(runner, happi_cfg):
     assert_match_expected(result, expected_output.split('\n'))
 
 
-@pytest.mark.parametrize("from_user, expected_output", [
+@pytest.mark.parametrize("from_user, expected_output, expected_return", [
     # Test add item - succeeding
     pytest.param('\n'.join(['HappiItem', 'happi_nname', 'device_class',
                             '', '', 'Y', 'docs', 'y']), (
@@ -282,12 +278,13 @@ def test_search_json(runner, happi_cfg):
         'Selecting value: True',
         'Enter value for documentation, enforce=str [optional]: docs',
         'Selecting value: docs',
-        'Please confirm the device info is correct [y/N]: y',
-        ' - INFO -  Adding device',
-        ' - INFO -  Storing device HappiItem (name=happi_nname) ...',
+        'Please confirm the item info is correct [y/N]: y',
+        ' - INFO -  Adding item',
+        ' - INFO -  Storing item HappiItem (name=happi_nname) ...',
         ' - INFO -  Adding / Modifying information for happi_nname ...',
         ' - INFO -  HappiItem HappiItem (name=happi_nname) has been '
         'succesfully added to the database'),
+        0
     ),
     # Test add item - aborting
     pytest.param('\n'.join(['HappiItem', 'happi_name2', 'device_class',
@@ -312,15 +309,17 @@ def test_search_json(runner, happi_cfg):
         'Selecting value: True',
         'Enter value for documentation, enforce=str [optional]: docs',
         'Selecting value: docs',
-        'Please confirm the device info is correct [y/N]: N',
+        'Please confirm the item info is correct [y/N]: N',
         ' - INFO -  Aborting'),
+        0
     ),
     # Test add item - invalid container
     pytest.param('HappiInvalidItem', (
         'Please select a container, or press enter for generic '
         'Ophyd Device container: ', 'OphydItem', 'HappiItem', '',
         'Selection [OphydItem]: HappiInvalidItem',
-        ' - INFO -  Invalid device container HappiInvalidItem'),
+        'Error: Invalid item container HappiInvalidItem'),
+        1
     ),
     # Test add item - no reponse, not an optional field,
     # invalid value, add OphydItem
@@ -351,20 +350,27 @@ def test_search_json(runner, happi_cfg):
         'Enter value for prefix, enforce=str: ',
         'Enter value for prefix, enforce=str: some_prefix',
         'Selecting value: some_prefix',
-        'Please confirm the device info is correct [y/N]: y',
-        ' - INFO -  Adding device',
-        ' - INFO -  Storing device OphydItem (name=ophyd_name) ...',
+        'Please confirm the item info is correct [y/N]: y',
+        ' - INFO -  Adding item',
+        ' - INFO -  Storing item OphydItem (name=ophyd_name) ...',
         ' - INFO -  Adding / Modifying information for ophyd_name ...',
         ' - INFO -  HappiItem OphydItem (name=ophyd_name) has been '
         'succesfully added to the database'
          ),
+         0
     ),
     ], ids=["add_succeeding", "add_aborting",
             "add_invalid_container", "add_not_optional_field"])
-def test_add_cli(from_user, expected_output, caplog, runner, happi_cfg):
+def test_add_cli(
+    from_user: str,
+    expected_output: Tuple[str, ...],
+    expected_return: int,
+    happi_cfg: str,
+    runner: CliRunner
+):
     result = runner.invoke(happi_cli, ['--path', happi_cfg, 'add'],
                            input=from_user)
-    assert_match_expected(result, expected_output)
+    assert_match_expected(result, expected_output, expected_return)
 
 
 @pytest.mark.parametrize("from_user, expected_output", [
@@ -384,20 +390,36 @@ def test_add_cli(from_user, expected_output, caplog, runner, happi_cfg):
         "Selecting value: {'name': 'my_name'}", 'Enter value for active, '
         'enforce=bool [Y/n]: ', 'Selecting value: True',
         'Enter value for documentation, enforce=str [docs]: ',
-        'Selecting value: docs', 'Please confirm the device info is '
-        'correct [y/N]: y', ' - INFO -  Adding device',
-        ' - INFO -  Storing device HappiItem (name=happi_new_name) ...',
+        'Selecting value: docs', 'Please confirm the item info is '
+        'correct [y/N]: y', ' - INFO -  Adding item',
+        ' - INFO -  Storing item HappiItem (name=happi_new_name) ...',
         ' - INFO -  Adding / Modifying information for happi_new_name ...',
         ' - INFO -  HappiItem HappiItem (name=happi_new_name) has been '
         'succesfully added to the database', ''], id="clone_succeeding",
     )])
-def test_add_clone(from_user, expected_output, happi_cfg, runner):
-    device_info = '\n'.join(['HappiItem', 'happi_name', 'device_class',
-                             "['arg1', 'arg2']", 'name', 'my_name', '',
-                             'Y', 'docs', 'y'])
-    # add device first
+def test_add_clone(
+    from_user: str,
+    expected_output: Tuple[str, ...],
+    happi_cfg: str,
+    runner: CliRunner
+):
+    item_info = "\n".join(
+        [
+            "HappiItem",
+            "happi_name",
+            "device_class",
+            "['arg1', 'arg2']",
+            "name",
+            "my_name",
+            "",
+            "Y",
+            "docs",
+            "y",
+        ]
+    )
+    # add item first
     add_result = runner.invoke(happi_cli, ['--path', happi_cfg, 'add'],
-                               input=device_info)
+                               input=item_info)
     assert add_result.exit_code == 0
 
     clone_result = runner.invoke(
@@ -407,7 +429,7 @@ def test_add_clone(from_user, expected_output, happi_cfg, runner):
     assert_match_expected(clone_result, expected_output)
 
 
-def test_add_clone_device_not_found(happi_cfg, runner):
+def test_add_clone_item_not_found(happi_cfg: str, runner: CliRunner):
     result = runner.invoke(
         happi_cli,
         ['--verbose', '--path', happi_cfg, 'add', '--clone', 'happi_name']
@@ -427,19 +449,37 @@ def test_add_clone_device_not_found(happi_cfg, runner):
      [False, 'yes'])
     ]
 )
-def test_edit(from_user, fields, values, happi_cfg, runner, client):
-    device_info = '\n'.join(['HappiItem', 'happi_name', 'device_class',
-                             "['arg1', 'arg2']", 'name', 'my_name', '',
-                             'Y', 'docs', 'y'])
-    # add device first
+def test_edit(
+    from_user: List[str],
+    fields: List[str],
+    values: List[Any],
+    client: happi.client.Client,
+    happi_cfg: str,
+    runner: CliRunner
+):
+    item_info = "\n".join(
+        [
+            "HappiItem",
+            "happi_name",
+            "device_class",
+            "['arg1', 'arg2']",
+            "name",
+            "my_name",
+            "",
+            "Y",
+            "docs",
+            "y",
+        ]
+    )
+    # add item first
     add_result = runner.invoke(
         happi_cli,
         ['--verbose', '--path', happi_cfg, 'add'],
-        input=device_info
+        input=item_info
     )
     assert add_result.exit_code == 0
 
-    # try edit the previous added device
+    # try edit the previous added item
     edit_result = runner.invoke(
         happi_cli,
         ['--path', happi_cfg, 'edit', 'happi_name', *from_user],
@@ -460,7 +500,7 @@ def test_edit(from_user, fields, values, happi_cfg, runner, client):
     ["name=2"],  # bad value for name
     ["kwargs={\'str\':\'beh\'}"],  # bad key in kwarg
 ])
-def test_bad_edit(edit_args, runner, happi_cfg):
+def test_bad_edit(edit_args: List[str], happi_cfg: str, runner: CliRunner):
     # Test invalid field, note the name is changed to new_name
     bad_edit_result = runner.invoke(
         happi_cli,
@@ -469,16 +509,32 @@ def test_bad_edit(edit_args, runner, happi_cfg):
     assert bad_edit_result.exit_code == 1
 
 
-def test_load(caplog, client, happi_cfg, runner):
-    device_info = '\n'.join(['HappiItem', 'happi_name',
-                             'types.SimpleNamespace', '', 'name', 'my_name',
-                             '', 'y', 'docs', 'y'])
-    # add device first
+def test_load(
+    caplog: pytest.LogCaptureFixture,
+    client: happi.client.Client,
+    happi_cfg: str,
+    runner: CliRunner
+):
+    item_info = "\n".join(
+        [
+            "HappiItem",
+            "happi_name",
+            "types.SimpleNamespace",
+            "",
+            "name",
+            "my_name",
+            "",
+            "y",
+            "docs",
+            "y",
+        ]
+    )
+    # add item first
     add_result = runner.invoke(happi_cli, ['--path', happi_cfg, 'add'],
-                               input=device_info)
+                               input=item_info)
     assert add_result.exit_code == 0
 
-    # try to load the device
+    # try to load the item
     devices = {}
     devices['happi_name'] = client.load_device(name='happi_name')
     with mock.patch.object(IPython, 'start_ipython') as m:
@@ -490,7 +546,7 @@ def test_load(caplog, client, happi_cfg, runner):
         assert "Creating shell with devices" in caplog.text
 
 
-def test_update(happi_cfg, runner):
+def test_update(happi_cfg: str, runner: CliRunner):
     new = """[ {
         "_id": "TST_BASE_PIM2",
         "active": true,
@@ -518,7 +574,7 @@ def test_update(happi_cfg, runner):
     result = runner.invoke(happi_cli, ["--path", happi_cfg, "update", new])
     assert result.exit_code == 0
     client = happi.client.Client.from_config(cfg=happi_cfg)
-    item = client.find_device(name="tst_base_pim2")
+    item = client.find_item(name="tst_base_pim2")
     assert item["from_update"]
 
 
@@ -553,10 +609,75 @@ def test_update(happi_cfg, runner):
         'Include entry from tst_base_pim: system = "diagnostic"? [Y/n]: n',
         'Include entry from tst_base_pim: y = "40.0"? [Y/n]: n',
         'Include entry from tst_base_pim: z = "3.0"? [Y/n]: n', '',
-        '----------Amend Entries-----------', 'Save final device? [y/N]: ',
+        '----------Amend Entries-----------', 'Save final item? [y/N]: ',
         ''], id="transfer_succeeding",
     )])
-def test_transfer_cli(from_user, expected_output, happi_cfg, runner):
+def test_transfer_cli(
+    from_user: str,
+    expected_output: List[str],
+    happi_cfg: str,
+    runner: CliRunner
+):
     results = runner.invoke(happi_cli, ['--path', happi_cfg, 'transfer',
                             'tst_base_pim', 'OphydItem'], input=from_user)
     assert_match_expected(results, expected_output)
+
+
+def arg_variants(variants: Tuple[Tuple[Tuple[str]]]):
+    """
+    Collapse argument variants into all possible combinations.
+    """
+    for arg_set in itertools.product(*variants):
+        yield functools.reduce(
+            lambda x, y: x+y,
+            arg_set,
+            )
+
+
+benchmark_arg_variants = (
+    (('--duration', '0.1'), ('--iterations', '5')),
+    (('--wait-connected',), ()),
+    (('--tracebacks',), ()),
+    (('--sort-key', 'name'), ()),
+    (('--glob', '*pim'), ('--regex', '.*pim'), ()),
+)
+
+
+@pytest.mark.parametrize(
+    "args", tuple(arg_variants(benchmark_arg_variants))
+)
+def test_benchmark_cli(runner: CliRunner, happi_cfg: str, args: Tuple[str]):
+    # Make sure the benchmark can complete in some form with valid inputs
+    result = runner.invoke(
+        happi_cli,
+        ['--path', happi_cfg, 'benchmark'] + list(args),
+    )
+    assert_in_expected(
+        result,
+        'Benchmark completed successfully'
+    )
+
+
+profile_arg_variants = (
+    (('--database',), ('--import',), ('--object', ),
+     ('--all', ), ('--import', '--object')),
+    (('--profiler', 'pcdsutils'), ('--profiler', 'cprofile'), ()),
+    (('--glob', '*pim'), ('--regex', '.*pim'), ()),
+)
+
+
+@pytest.mark.parametrize(
+    "args", tuple(arg_variants(profile_arg_variants))
+)
+def test_profile_cli(runner: CliRunner, happi_cfg: str, args: Tuple[str]):
+    # Make sure the profile can complete in some form with valid inputs
+    if 'pcdsutils' in args and not test_line_prof:
+        pytest.skip('Missing pcdsutils or line_profiler.')
+    result = runner.invoke(
+        happi_cli,
+        ['--path', happi_cfg, 'profile'] + list(args),
+    )
+    assert_in_expected(
+        result,
+        'Profile completed successfully'
+    )
