@@ -1,36 +1,39 @@
+from typing import Any
+
 import pytest
 
-from happi import Device, EntryInfo, cache
+from happi import EntryInfo, OphydItem, cache
+from happi.item import HappiItem
 from happi.loader import fill_template, from_container, load_devices
 from happi.utils import create_alias
 
 
-class TimeDevice(Device):
+class TimeDevice(OphydItem):
     days = EntryInfo("Number of days", enforce=int)
 
 
-def test_fill_template(device):
+def test_fill_template(item: OphydItem):
     # Check that we can properly render a template
     template = "{{name}}"
-    assert device.name == fill_template(template, device)
+    assert item.name == fill_template(template, item)
     # Check that we can enforce a type
     template = '{{active}}'
-    active = fill_template(template, device, enforce_type=True)
+    active = fill_template(template, item, enforce_type=True)
     assert isinstance(active, bool)
     # Check that we will convert a more complex template
     template = '{{name|length()}}'
-    text_len = fill_template(template, device, enforce_type=False)
-    assert len(device.name) == int(text_len)
+    text_len = fill_template(template, item, enforce_type=False)
+    assert len(item.name) == int(text_len)
     # Check that we can handle non-jinja template
     template = "blah"
-    assert template == fill_template(template, device, enforce_type=True)
+    assert template == fill_template(template, item, enforce_type=True)
     # Check that we do not enforce a NoneType
     template = "{{detailed_screen}}"
-    assert fill_template(template, device, enforce_type=True) == ""
+    assert fill_template(template, item, enforce_type=True) == ""
 
 
 def test_from_container():
-    # Create a datetime device
+    # Create a datetime item
     d = TimeDevice(name='test', prefix='Tst:This:1', beamline='TST',
                    device_class='datetime.timedelta', args=list(), days=10,
                    kwargs={'days': '{{days}}', 'seconds': 30})
@@ -41,7 +44,7 @@ def test_from_container():
 
 
 def test_caching():
-    # Create a datetime device
+    # Create a datetime item
     d = TimeDevice(name='test', prefix='Tst:This:2', beamline='TST',
                    device_class='types.SimpleNamespace', args=list(), days=10,
                    kwargs={'days': '{{days}}', 'seconds': 30})
@@ -52,7 +55,7 @@ def test_caching():
     # Modify md and check we see a reload
     d.days = 12
     assert id(td) != id(from_container(d, use_cache=True))
-    # Check with a device where metadata is unavailable
+    # Check with an item where metadata is unavailable
     d = TimeDevice(name='test', prefix='Tst:Delta:3', beamline='TST',
                    device_class='datetime.timedelta', args=list(), days=10,
                    kwargs={'days': '{{days}}', 'seconds': 30})
@@ -62,11 +65,11 @@ def test_caching():
 
 
 def test_add_md():
-    d = Device(name='test', prefix='Tst:This:3',
-               beamline="TST", args=list(),
-               device_class="happi.Device")
+    d = HappiItem(name='test', prefix='Tst:This:3',
+                  beamline="TST", args=list(),
+                  device_class="happi.HappiItem")
     obj = from_container(d, attach_md=True)
-    assert obj.md.beamline == 'TST'
+    assert obj.md.extraneous['beamline'] == 'TST'
     assert obj.md.name == 'test'
 
 
@@ -91,7 +94,7 @@ def test_add_md():
         pytest.param(True, id="load_times")
     ],
 )
-def test_load_devices(threaded: bool, post_load, include_load_time: bool):
+def test_load_devices(threaded: bool, post_load: Any, include_load_time: bool):
     # Create a bunch of devices to load
     devs = [TimeDevice(name='test_1', prefix='Tst1:This', beamline='TST',
                        device_class='datetime.timedelta', args=list(), days=10,
@@ -102,8 +105,8 @@ def test_load_devices(threaded: bool, post_load, include_load_time: bool):
             TimeDevice(name='test_3', prefix='Tst3:This', beamline='TST',
                        device_class='datetime.timedelta', args=list(), days=10,
                        kwargs={'days': '{{days}}', 'seconds': 30}),
-            Device(name='bad', prefix='Not:Here', beamline='BAD',
-                   device_class='non.existant')]
+            HappiItem(name='bad', prefix='Not:Here', beamline='BAD',
+                      device_class='non.existant')]
     # Load our devices
     space = load_devices(
         *devs,
@@ -119,3 +122,30 @@ def test_load_devices(threaded: bool, post_load, include_load_time: bool):
     import datetime
     assert space.test_1 == datetime.timedelta(days=10, seconds=30)
     assert isinstance(space.bad, ImportError)
+
+
+def test_filter_kwargs(item_jinja: OphydItem):
+    blanks = ['blank_bool', 'blank_list', 'blank_str', 'blank_none']
+
+    # default behavior, allow individuals to decide
+    item_jinja._info_attrs['kwargs'].include_default_as_kwarg = True
+    dev = from_container(item_jinja, use_cache=False)
+
+    # basic jinja template filling test.
+    # only kwargs get attrs in SimpleNamespace
+    assert dev.loc == 'LOC'
+
+    # if there is no correspoding EntryInfo, this is a piece of metadata
+    # type cannot be matched and value will be returned as string
+    assert dev.blank == 'None'
+    assert getattr(dev, 'blank_exclude', 'DNE') == 'DNE'
+    for bl in blanks:
+        assert getattr(dev, bl, 'DNE') == item_jinja._info_attrs[bl].default
+
+    item_jinja._info_attrs['kwargs'].include_default_as_kwarg = False
+    filtered_dev = from_container(item_jinja, use_cache=False)
+
+    assert filtered_dev.blank == 'None'
+    assert getattr(filtered_dev, 'blank_exclude', 'DNE') == 'DNE'
+    for bl in blanks:
+        assert getattr(filtered_dev, bl, 'DNE') == 'DNE'
